@@ -1,9 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import base64
-import json
 import pytest
-import queue
 import tuxsuite.build
 import requests
 import tuxsuite.exceptions
@@ -65,6 +63,16 @@ class TestPostRequest:
                 url="http://foo.bar.com/bad_request", headers=headers, request=request
             )
 
+    def test_post_request_error_request(self, post, response):
+        request = {"a": "b"}
+        response.status_code = 400
+        response._content = b'{"tuxbuild_status": "a", "error": "b"}'
+
+        with pytest.raises(tuxsuite.exceptions.BadRequest):
+            tuxsuite.build.post_request(
+                url="http://foo.bar.com/bad_request", headers=headers, request=request
+            )
+
 
 class TestGetRequest:
     def test_get_request_pass(self, get, response, mocker):
@@ -111,6 +119,14 @@ class TestGetRequest:
                 url="http://foo.bar.com/connection_failure", headers=headers
             )
 
+    def test_get_request_unknown_request(self, get, response):
+        response.status_code = 404
+
+        with pytest.raises(tuxsuite.exceptions.URLNotFound):
+            tuxsuite.build.get_request(
+                url="http://foo.bar.com/bad_request", headers=headers
+            )
+
 
 @pytest.fixture
 def start_time():
@@ -136,6 +152,216 @@ def sleep(mocker, start_time):
     return mocker.patch("time.sleep", side_effect=mock_sleep)
 
 
+@pytest.fixture
+def bitbake_attrs():
+    return {
+        "group": "tuxsuite",
+        "project": "unittests",
+        "token": "test_token",
+        "tuxapi_url": "http://tuxapi",
+        "kbapi_url": "http://tuxapi",
+        "data": {
+            "distro": "rpb",
+            "envsetup": "setup-environment",
+            "machine": "dragonboard-845c",
+            "sources": {
+                "branch": "dunfell",
+                "manifest": "default.xml",
+                "url": "https://github.com/96boards/oe-rpb-manifest.git",
+            },
+            "target": "rpb-console-image rpb-console-image-test rpb-desktop-image rpb-desktop-image-test",
+        },
+    }
+
+
+class TestWatch:
+    @staticmethod
+    def watch(obj):
+        states = []
+        for state in obj.watch():
+            states.append(state)
+        return states
+
+
+@pytest.fixture
+def bitbake(bitbake_attrs):
+    b = tuxsuite.build.Bitbake(**bitbake_attrs)
+    b.uid = "myuid"
+    return b
+
+
+class TestBitbake:
+    def test_build_definition(self, bitbake):
+        expected_data = {
+            "distro": "rpb",
+            "envsetup": "setup-environment",
+            "machine": "dragonboard-845c",
+            "sources": {
+                "branch": "dunfell",
+                "manifest": "default.xml",
+                "url": "https://github.com/96boards/oe-rpb-manifest.git",
+            },
+            "target": "rpb-console-image rpb-console-image-test rpb-desktop-image rpb-desktop-image-test",
+            "artifacts": [],
+            "environment": {},
+            "local_conf": [],
+            "bblayers_conf": [],
+            "container": bitbake.build_definition.container,
+        }
+        assert bitbake.generate_build_request() == expected_data
+
+    def test_submit_build(self, bitbake, bitbake_attrs, mocker):
+        post_request = mocker.patch("tuxsuite.build.post_request")
+        api_build_url = (
+            bitbake_attrs["tuxapi_url"]
+            + "/v1/groups/tuxsuite/projects/unittests/oebuilds"
+        )
+
+        bitbake.build()
+        post_request.assert_called_with(
+            api_build_url,
+            mocker.ANY,
+            {
+                "distro": "rpb",
+                "envsetup": "setup-environment",
+                "machine": "dragonboard-845c",
+                "sources": {
+                    "branch": "dunfell",
+                    "manifest": "default.xml",
+                    "url": "https://github.com/96boards/oe-rpb-manifest.git",
+                },
+                "target": "rpb-console-image rpb-console-image-test rpb-desktop-image rpb-desktop-image-test",
+                "artifacts": [],
+                "environment": {},
+                "local_conf": [],
+                "bblayers_conf": [],
+                "container": bitbake.build_definition.container,
+            },
+        )
+
+
+class TestBitbakeWatch(TestWatch):
+    @pytest.fixture(autouse=True)
+    def set_bitbake_key(self, bitbake):
+        bitbake.uid = "0123456789"
+        return bitbake
+
+    @pytest.fixture
+    def bitbake_statuses(self):
+        return [
+            {
+                "state": "provisioning",
+                "result": "unknown",
+                "status_message": "Queued",
+            },
+            {
+                "state": "running",
+                "result": "unknown",
+                "status_message": "Building ...",
+            },
+            {
+                "state": "running",
+                "result": "unknown",
+                "status_message": "Building ...",
+            },
+            {
+                "state": "finished",
+                "result": "pass",
+                "warnings_count": 0,
+                "errors_count": 0,
+                "status_message": "Finished ...",
+            },
+        ]
+
+    @pytest.fixture(autouse=True)
+    def get_request(self, mocker, bitbake_statuses):
+        return mocker.patch("tuxsuite.build.get_request", side_effect=bitbake_statuses)
+
+    def test_watch_pass(self, bitbake):
+        str(bitbake)
+        watch = iter(bitbake.watch())
+        s1 = next(watch)
+        assert s1.state == "provisioning"
+
+        s2 = next(watch)
+        assert s2.state == "running"
+
+        bitbake.status["warnings_count"] = 0
+        s3 = next(watch)
+        assert s3.state == "finished"
+
+    def test_watch_pass_warnings(self, bitbake, bitbake_statuses):
+        bitbake_statuses[-1]["warnings_count"] = 5
+
+        state = self.watch(bitbake)[-1]
+        assert "Pass (5 warnings)" in state.message
+        assert state.warnings == 5
+
+    def test_watch_pass_one_warning(self, bitbake, bitbake_statuses):
+        bitbake_statuses[-1]["warnings_count"] = 1
+
+        state = self.watch(bitbake)[-1]
+        assert "Pass (1 warning)" in state.message
+        assert state.warnings == 1
+
+    def test_watch_fail(self, bitbake, bitbake_statuses):
+        bitbake_statuses[-1]["result"] = "fail"
+        bitbake_statuses[-1]["errors_count"] = 5
+
+        state = self.watch(bitbake)[-1]
+        assert "Fail (1 error)" in state.message
+        assert state.errors == 5
+
+    def test_watch_fail_1_error(self, bitbake, bitbake_statuses):
+        bitbake_statuses[-1]["result"] = "fail"
+        bitbake_statuses[-1]["errors_count"] = 1
+
+        state = self.watch(bitbake)[-1]
+        assert "Fail (1 error)" in state.message
+        assert state.errors == 1
+
+    def test_watch_warnings(self, bitbake):
+        watch = iter(bitbake.watch())
+        s1 = next(watch)
+        assert s1.state == "provisioning"
+
+        s2 = next(watch)
+        assert s2.state == "running"
+
+        bitbake.status["warnings_count"] = 1
+        s3 = next(watch)
+        assert s3.state == "finished"
+
+    def test_retries_on_errors(self, bitbake, mocker, bitbake_statuses):
+        bitbake_statuses[-1]["build_status"] = None
+        bitbake_statuses[-1]["state"] = "finished"
+        bitbake_statuses[-1]["result"] = "error"
+        bitbake_statuses[-1]["status_message"] = "the infrastructure failed"
+        bitbake_statuses.append(bitbake_statuses[0])
+        bitbake_statuses.append(bitbake_statuses[1])
+        bitbake_statuses.append(bitbake_statuses[2])
+        bitbake_statuses.append(bitbake_statuses[0])
+        bitbake_statuses.append(bitbake_statuses[1])
+        bitbake_statuses.append(bitbake_statuses[2])
+        build_build = mocker.patch("tuxsuite.build.Bitbake.build")
+
+        assert build_build.call_count == 0
+
+
+class TestBakeWait:
+    def test_wait(self, bitbake, mocker):
+        watch = mocker.patch("tuxsuite.build.Bitbake.watch")
+        bitbake.wait()
+        assert watch.call_count > 0
+
+    def test_wait_returns_last_state(self, bitbake, mocker):
+        watch = mocker.patch("tuxsuite.build.Bitbake.watch")
+        first = mocker.MagicMock()
+        last = mocker.MagicMock()
+        watch.return_value = [first, last]
+        assert bitbake.wait() is last
+
+
 @pytest.fixture
 def build_attrs():
     return {
@@ -422,15 +648,6 @@ class TestBuild:
         )
 
 
-class TestWatch:
-    @staticmethod
-    def watch(obj):
-        states = []
-        for state in obj.watch():
-            states.append(state)
-        return states
-
-
 class TestBuildWatch(TestWatch):
     @pytest.fixture(autouse=True)
     def set_build_key(self, build):
@@ -547,57 +764,6 @@ class TestBuildWatch(TestWatch):
         assert state.status is None
         assert "the infrastructure failed" in state.message
 
-    def test_retries_on_errors(self, build, mocker, build_statuses):
-        build_statuses[-1]["build_status"] = None
-        build_statuses[-1]["tuxbuild_status"] = "error"
-        build_statuses[-1]["state"] = "finished"
-        build_statuses[-1]["result"] = "error"
-        build_statuses[-1]["status_message"] = "the infrastructure failed"
-        build_statuses.append(build_statuses[0])
-        build_statuses.append(build_statuses[1])
-        build_statuses.append(build_statuses[2])
-        build_statuses.append(build_statuses[0])
-        build_statuses.append(build_statuses[1])
-        build_statuses.append(build_statuses[2])
-        build_build = mocker.patch("tuxsuite.build.Build.build")
-
-        states = self.watch(build)
-        assert build_build.call_count == 2
-        assert [state.state for state in states] == [
-            "queued",
-            "building",
-            "error",
-            "queued",
-            "building",
-            "error",
-            "queued",
-            "building",
-            "error",
-        ]
-
-    def test_retry_succeeds_in_the_second_attempt(self, build, mocker, build_statuses):
-        complete = build_statuses[-1].copy()
-        build_statuses[-1]["build_status"] = None
-        build_statuses[-1]["tuxbuild_status"] = "error"
-        build_statuses[-1]["state"] = "finished"
-        build_statuses[-1]["result"] = "error"
-        build_statuses[-1]["status_message"] = "the infrastructure failed"
-        build_statuses.append(build_statuses[0])
-        build_statuses.append(build_statuses[1])
-        build_statuses.append(complete)
-        build_build = mocker.patch("tuxsuite.build.Build.build")
-
-        states = self.watch(build)
-        assert build_build.call_count == 1
-        assert [state.state for state in states] == [
-            "queued",
-            "building",
-            "error",
-            "queued",
-            "building",
-            "complete",
-        ]
-
     def test_from_queued_directly_to_completed(self, build, mocker, build_statuses):
         build_statuses.pop(1)
         states = self.watch(build)
@@ -670,102 +836,6 @@ def builds():
     ]
 
 
-@pytest.fixture
-def build_set(build_attrs, builds):
-    return tuxsuite.build.BuildSet(
-        builds,
-        group=build_attrs["group"],
-        project=build_attrs["project"],
-        git_repo=build_attrs["git_repo"],
-        git_ref=build_attrs["git_ref"],
-        kbapi_url=build_attrs["kbapi_url"],
-        tuxapi_url=build_attrs["tuxapi_url"],
-        token=build_attrs["token"],
-    )
-
-
-class TestBuildSet:
-    def test_expand_spec(self, build_set):
-        assert len(build_set.builds) == 5
-        assert build_set
-        assert build_set.builds[0].git_repo is not None
-
-    def test_updates_builds_with_returned_data(self, build_set, post, response):
-        builds = build_set.builds
-        server_data = [
-            {
-                "uid": "myuid1",
-                "download_url": "https://builds.example/com/00000000000",
-            },
-            {
-                "uid": "myuid2",
-                "download_url": "https://builds.example/com/11111111111",
-            },
-            {
-                "uid": "myuid3",
-                "download_url": "https://builds.example/com/22222222222",
-            },
-            {
-                "uid": "myuid4",
-                "download_url": "https://builds.example/com/33333333333",
-            },
-            {
-                "uid": "myuid5",
-                "download_url": "https://builds.example/com/44444444444",
-            },
-        ]
-        data = [
-            dict(**builds[i].generate_build_request()[0], **server_data[i])
-            for i in range(len(builds))
-        ]
-        response._content = json.dumps(list(reversed(data))).encode()
-        build_set.build()
-
-        assert builds[0].uid == "myuid1"
-        assert builds[1].uid == "myuid2"
-        assert builds[2].uid == "myuid3"
-        assert builds[3].uid == "myuid4"
-        assert builds[4].uid == "myuid5"
-
-
-class TestBuildSetWatch(TestWatch):
-    def test_watch(self, build_set, mocker):
-        build_watch = mocker.patch("tuxsuite.build.Build.watch")
-        state1 = mocker.MagicMock()
-        state1.final = False
-        state2 = mocker.MagicMock()
-        state2.final = True
-        build_watch.return_value = [state1, state2]
-
-        states = self.watch(build_set)
-        assert len(states) == 2 * len(build_set.builds)
-
-    def test_watch_when_threads_crash(self, build_set, mocker, capsys):
-        build_watch = mocker.patch("tuxsuite.build.Build.watch")
-        build_watch.side_effect = RuntimeError("BOOM")
-        mocker.patch("queue.Queue.get", side_effect=queue.Empty)
-
-        self.watch(build_set)
-
-        # if the test reaches this point, it means BuildSet.watch didn't get
-        # stuck waiting forever.
-        _, err = capsys.readouterr()
-        assert "ERROR for build" in err
-
-
-class TestBuildSetWait:
-    def test_wait(self, build_set, mocker):
-        watch = mocker.patch("tuxsuite.build.BuildSet.watch")
-        state1 = mocker.MagicMock()
-        state1.final = False
-        state2 = mocker.MagicMock()
-        state2.final = True
-        watch.return_value = [state1, state1, state2, state2]
-
-        results = build_set.wait()
-        assert results == [state2, state2]
-
-
 @pytest.fixture
 def results(build_attrs):
     return tuxsuite.build.Results(
@@ -864,3 +934,55 @@ class TestHandlePatch:
         assert isinstance(type(result[0]), type(str))
         assert result[0] == url
         assert base64.b64decode(result[1]) == b""
+
+
+@pytest.fixture
+def test_attrs():
+    return {
+        "group": "tuxsuite",
+        "project": "unittests",
+        "device": "qemu-x86_64",
+        "kernel": "https://storage.tuxboot.com/x86_64/bzImage",
+        "tests": ["boot", "ltp-smoke"],
+        "token": "test_token",
+        "kbapi_url": "http://test/foo",
+        "tuxapi_url": "http://tuxapi",
+    }
+
+
+@pytest.fixture
+def test(test_attrs):
+    t = tuxsuite.build.Test(**test_attrs)
+    t.uid = "myuid"
+    return t
+
+
+class TestTest:
+    def test_test_str(self, test):
+        assert type(test.tests) == list
+        print(test)
+
+    def test_submit_test(self, test, test_attrs, mocker):
+        post_request = mocker.patch("tuxsuite.build.post_request")
+        api_test_url = (
+            test_attrs["tuxapi_url"] + "/v1/groups/tuxsuite/projects/unittests/tests"
+        )
+
+        test.test()
+        post_request.assert_called_with(
+            api_test_url,
+            mocker.ANY,
+            {
+                "kernel": test_attrs["kernel"],
+                "device": test_attrs["device"],
+                "ap_romfw": None,
+                "mcp_fw": None,
+                "mcp_romfw": None,
+                "parameters": [],
+                "rootfs": None,
+                "scp_fw": None,
+                "scp_romfw": None,
+                "fip": None,
+                "tests": test_attrs["tests"],
+            },
+        )
