@@ -5,7 +5,7 @@
 This is the tuxsuite module.
 """
 
-__version__ = "0.42.2"
+__version__ = "0.43.2"
 
 
 from . import build
@@ -53,10 +53,10 @@ class Build(Configurable, build.Build):
     """
 
 
-class BuildSet(Configurable, build.BuildSet):
+class Bitbake(Configurable, build.Bitbake):
     """
-    This class represents a set of builds that can be triggered and watched
-    simultaneously.
+    This class represents individual builds. It should be used to trigger
+    builds, and optionally wait for them to finish.
     """
 
 
