@@ -2,18 +2,14 @@
 
 from attr import attrs, attrib
 import base64
-import concurrent.futures
 import hashlib
 import json
-import queue
 import mimetypes
 import os
 import random
 import re
-import sys
 import tarfile
 import time
-import traceback
 import tuxsuite.exceptions
 import tuxsuite.requests
 from tuxsuite import __version__
@@ -322,8 +318,6 @@ class Build(HeadersMixin):
                         icon="🔧",
                         cli_color="yellow",
                     )
-                    plan = self.status.get("plan", None)
-                    self.build(plan=plan)
                 elif state == "finished":
                     break
                 else:
@@ -431,88 +425,6 @@ class Build(HeadersMixin):
         return self._get_field("status_message")
 
 
-class BuildSet(HeadersMixin):
-    def __init__(self, builds, **kwargs):
-        self.builds = []
-        for item in builds:
-            data = kwargs.copy()
-            data.update(item)
-            self.builds.append(Build(**data))
-        self.token = kwargs["token"]
-        self.tuxapi_url = kwargs["tuxapi_url"]
-        self.project = kwargs["project"]
-        self.group = kwargs["group"]
-
-    def build(self):
-        """Submit the build request"""
-
-        data = {"builds": [], "patches": {}}
-        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/builds".format(
-            self.group, self.project
-        )
-        for build_object in self.builds:
-            build_entry, patch = build_object.generate_build_request()
-            data["builds"].append(build_entry)
-            data["patches"].update(patch)
-        json_data = post_request(url, self.headers, data)
-
-        builds_by_client_token = {b.client_token: b for b in self.builds}
-        for build_config in json_data:
-            build = builds_by_client_token[build_config["client_token"]]
-            build.status = build_config
-            build.build_data = build_config["download_url"]
-            build.uid = build_config["uid"]
-
-    def __watch_build__(self, build, results):
-        for state in build.watch():
-            results.put(state)
-
-    def watch(self):
-        results = queue.Queue()
-        n = len(self.builds)
-
-        with concurrent.futures.ThreadPoolExecutor(max_workers=n) as executor:
-            futures = {}
-            for build in self.builds:
-                f = executor.submit(self.__watch_build__, build, results)
-                futures[build.uid] = f
-
-            finished = 0
-            while finished < n:
-                try:
-                    state = results.get(timeout=60)
-                except queue.Empty:
-                    if all([f.done() for f in futures.values()]):
-                        # Something went wrong: all threads finished but we
-                        # don't have enough final states yet.
-                        # Report any exceptions and don't wait forever
-                        for build, f in futures.items():
-                            exc = f.exception()
-                            if exc:
-                                tb = "".join(traceback.format_tb(exc.__traceback__))
-                                print(
-                                    f"ERROR for build {build}: {exc}\n{tb}",
-                                    file=sys.stderr,
-                                )
-                        break
-                else:
-                    yield state
-                    if state.final:
-                        finished += 1
-
-    def wait(self):
-        states = []
-        for s in self.watch():
-            if s.final:
-                states.append(s)
-        return states
-
-    @property
-    def status_list(self):
-        """Return a list of build status dictionaries"""
-        return [build.status for build in self.builds]
-
-
 class Plan(HeadersMixin):
     def __init__(self, config, **kwargs):
         self.config = config
@@ -548,6 +460,8 @@ class Plan(HeadersMixin):
             if cfg["build"] is not None:
                 data = self.args.copy()
                 data.update(cfg["build"])
+                if self.args.get("no_cache"):
+                    data["no_cache"] = True
                 builds.append(Build(**data))
             else:
                 builds.append(None)
@@ -786,6 +700,11 @@ class Plan(HeadersMixin):
                 states.append(s)
         return states
 
+    @property
+    def build_status_list(self):
+        """Return a list of build status dictionaries"""
+        return [b.status for b in self.builds]
+
 
 @attrs(kw_only=True)
 class Test(HeadersMixin):
@@ -999,3 +918,174 @@ class Results(HeadersMixin):
         all_results["tests"] = get_request(test_url, self.headers)
         all_results["plans"] = get_request(plan_url, self.headers)
         return (all_results, test_url)
+
+
+@attrs(kw_only=True, order=False)
+class Bitbake(HeadersMixin):
+    token = attrib()
+    tuxapi_url = attrib()
+    kbapi_url = attrib()
+    status = attrib(default={})
+    uid = attrib(default=None)
+    group = attrib()
+    project = attrib()
+    data = attrib(default={})
+    build_definition = attrib(default={})
+    build_data = attrib(default=None)
+
+    @attrs(kw_only=True, order=False)
+    class BuildDefinition:
+        target = attrib()
+        envsetup = attrib()
+        distro = attrib()
+        machine = attrib()
+        sources = attrib(default={})
+        artifacts = attrib(default=[])
+        environment = attrib(default={})
+        local_conf = attrib(default=[])
+        bblayers_conf = attrib(default=[])
+        container = attrib(default="ubuntu-20.04")
+
+    def __attrs_post_init__(self):
+        self.build_definition = self.BuildDefinition(**(self.data))
+
+    def __str__(self):
+        return "{} with {} {} for {}: {} ".format(
+            self.build_definition.sources,
+            self.build_definition.distro,
+            self.build_definition.machine,
+            self.build_definition.target,
+            self.build_data,
+        )
+
+    def generate_build_request(self, plan=None):
+        """return a build data in a python dict"""
+        return self.build_definition.__dict__
+
+    def build(self, plan=None):
+        """Submit the build request"""
+        data = self.generate_build_request(plan=plan)
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/oebuilds".format(
+            self.group, self.project
+        )
+        self.status = post_request(url, self.headers, data)
+        self.build_data = self.status["download_url"]
+        self.uid = self.status["uid"]
+
+    def get_status(self):
+        """Fetches the build status and updates the values inside the build object"""
+        url = self.tuxapi_url + "/v1/groups/{}/projects/{}/oebuilds/{}".format(
+            self.group, self.project, self.uid
+        )
+        self.status = get_request(url, self.headers)
+
+    def watch(self, iterator=False):
+        """
+        Wait for the build to finish. Whenever the state changes, yielding the
+        new state).
+
+        Will timeout after `build_state_timeout` seconds.
+        """
+        waiting_states = {
+            "provisioning": BuildState(
+                build=self,
+                state="provisioning",
+                message="Provisioning",
+                icon="⚙️ ",
+                cli_color="white",
+            ),
+            "running": BuildState(
+                build=self,
+                state="running",
+                message="Running",
+                icon="🚀",
+                cli_color="cyan",
+            ),
+        }
+        previous_state = None
+        while True:
+            self.get_status()
+            state = self.status["state"]
+            if state != previous_state:
+                if state in waiting_states:
+                    yield waiting_states[state]
+                elif state == "finished":
+                    break
+            elif iterator:
+                yield
+
+            if not iterator:
+                delay = random.randrange(delay_status_update)
+                time.sleep(delay)
+            previous_state = state
+
+        errors = 0
+        warnings = 0
+        state = self.status["state"]
+        status = self.status["result"]
+
+        if status == "pass":
+            warnings = self.status["warnings_count"]
+            if warnings == 0:
+                icon = "🎉"
+                message = "Pass"
+                cli_color = "green"
+            else:
+                icon = "👾"
+                cli_color = "yellow"
+                if warnings == 1:
+                    message = "Pass (1 warning)"
+                else:
+                    message = "Pass ({} warnings)".format(warnings)
+        elif status == "fail":
+            icon = "👹"
+            cli_color = "bright_red"
+            errors = self.status["errors_count"]
+            message = "Fail (1 error)"
+            error_message = self.status.get("status_message")
+            if error_message:
+                message += f" with status message '{error_message}'"
+
+        finished = BuildState(
+            build=self,
+            state=state,
+            status=status,
+            final=True,
+            message=message,
+            icon=icon,
+            cli_color=cli_color,
+            warnings=warnings,
+            errors=errors,
+        )
+        yield finished
+
+    def wait(self):
+        state = None
+        for s in self.watch():
+            state = s
+        return state
+
+    def _get_field(self, field):
+        """Retrieve an individual field from status.json"""
+        self.get_status()
+        return self.status.get(field, None)
+
+    @property
+    def warnings_count(self):
+        """Get the warnings_count for the build"""
+        return int(self._get_field("warnings_count"))
+
+    @property
+    def errors_count(self):
+        """Get the errors_count for the build"""
+        return int(self._get_field("errors_count"))
+
+    @property
+    def build_status(self):
+        """Get the build_status for the build"""
+        return self._get_field("result")
+
+    @property
+    def status_message(self):
+        """Get the build_status for the build"""
+        return self._get_field("status_message")
