@@ -337,6 +337,7 @@ def common_options(required):
         "clang-11",
         "clang-12",
         "clang-13",
+        "clang-14",
         "clang-nightly",
         "clang-android",
         "rust",
@@ -509,6 +510,62 @@ description = (
 )
 
 
+@click.group(
+    help="Do an OE/Yocto build with bitbake like 'tuxsuite bake submit <build-definition.json>'",
+    short_help="Do an OE/Yocto build with bitbake like 'tuxsuite bake submit <build-definition.json>'",
+)
+def bake():
+    pass
+
+
+@bake.command(
+    help="Run a single OE build with 'tuxsuite bake submit <build-definition.json>'",
+    short_help="Run a single OE build with 'tuxsuite bake submit <build-definition.json>'",
+)
+@click.argument("build_definition", nargs=1)
+@click.option(
+    "--json-out",
+    help="Write json build status out to a named file path",
+    type=click.File("w", encoding="utf-8"),
+)
+def submit(build_definition, json_out=None):
+    try:
+        with open(os.path.abspath(build_definition)) as reader:
+            data = json_lib.load(reader)
+    except Exception:
+        sys.stderr.write(
+            f"Problem parsing {build_definition}, Is it a valid json file ?\n"
+        )
+        sys.exit(1)
+    try:
+        build = tuxsuite.Bitbake(data=data)
+    except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
+        error(e)
+    info(
+        "*** WARNING: BITBAKE SUPPORT IS EXPERIMENTAL ***\n"
+        "Building target: {} with bitbake from {} source with distro: {} machine: {} arguments".format(
+            build.build_definition.target,
+            build.build_definition.sources,
+            build.build_definition.distro,
+            build.build_definition.machine,
+        )
+    )
+    try:
+        build.build()
+        info("uid: {}".format(build.uid))
+    except tuxsuite.exceptions.BadRequest as e:
+        raise (click.ClickException(str(e)))
+
+    build_result = wait_for_object(build)
+    if json_out:
+        json_out.write(json_lib.dumps(build.status, sort_keys=True, indent=4))
+    if not build_result:
+        sys.exit(1)
+
+
+cli.add_command(bake)
+
+
 @cli.command(help=description, short_help="Run a single build.")
 @click.argument("action", metavar="[ACTION ...]", required=False)
 @common_options(required=["--target-arch", "--kconfig", "--toolchain"])
@@ -628,47 +685,44 @@ def build_set(
         build_params["patch_series"] = click.format_filename(patch_series)
 
     try:
-        build_set_config = tuxsuite.config.BuildSetConfig(set_name, tux_config)
-        build_set = tuxsuite.BuildSet(build_set_config.entries, **build_params)
+        plan_config = tuxsuite.config.BuildSetConfig(set_name, tux_config)
+        if not plan_config.plan:
+            warning("Empty plan, skipping")
+            return
+        plan = tuxsuite.Plan(plan_config, **build_params)
     except (AssertionError, tuxsuite.exceptions.TuxSuiteError) as e:
         error(e)
 
-    info("Building Linux Kernel build set {}".format(set_name))
-
     try:
-        build_set.build()
+        plan.submit()
     except tuxsuite.exceptions.BadRequest as e:
         raise (click.ClickException(str(e)))
 
-    if no_wait:
-        build_status_list = []
-        for build in build_set.builds:
-            format_result(build.status)
-            if json_out:
-                build_status_list.append(build.status)
-        if build_status_list:
-            json_out.write(json_lib.dumps(build_status_list, sort_keys=True, indent=4))
-        return
+    info("Building Linux Kernel build set {}".format(plan_config.name))
 
-    build_set_result = wait_for_object(build_set)
+    result = True
+
+    if no_wait:
+        for build in plan.builds:
+            format_result(build.status)
+    else:
+        result = wait_for_object(plan)
 
     if json_out:
-        json_out.write(json_lib.dumps(build_set.status_list, sort_keys=True, indent=4))
+        json_out.write(json_lib.dumps(plan.build_status_list, sort_keys=True, indent=4))
 
     if download:
-        for build in build_set.builds:
+        for build in plan.builds:
             tuxsuite.download.download(build, output_dir)
-
     if show_logs:
-        for build in build_set.builds:
+        for build in plan.builds:
             show_log(build, download, output_dir)
 
     if quiet:
-        for build in build_set.builds:
+        for build in plan.builds:
             print(build.build_data)
 
-    # If any of the builds did not pass, exit with exit code of 1
-    if not build_set_result:
+    if not result:
         sys.exit(1)
 
 
@@ -695,6 +749,7 @@ DEVICES = [
     "qemu-riscv",
     "qemu-riscv64",
     "qemu-s390",
+    "qemu-sh4",
     "qemu-sparc64",
     "qemu-x86_64",
 ]
@@ -708,6 +763,13 @@ TESTS = [
     "device-tree",
     "dvfs",
     "fwts",
+    "kselftest-gpio",
+    "kselftest-ipc",
+    "kselftest-ir",
+    "kselftest-kcmp",
+    "kselftest-kexec",
+    "kselftest-rseq",
+    "kselftest-rtc",
     "kunit",
     "lldb",
     "libjpeg-turbo",
@@ -1010,6 +1072,12 @@ plan_description = (
     is_flag=True,
     help="Prints build logs to stderr in case of warnings or errors",
 )
+@click.option(
+    "--no-cache",
+    default=False,
+    is_flag=True,
+    help="Build without using any compilation cache",
+)
 @click.option(
     "-n",
     "--no-wait",
